/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "importkradiorippersettings.h"

#include <QFile>
#include <QFileInfo>
#include <QTextStream>
#include <KConfig>
#include <KConfigGroup>
#include <KLocalizedString>
#include <KMessageBox>
#include <KStandardDirs>
#include <KTemporaryFile>
#include <KUrl>
#include "includekio.h"
#include "settings_general.h"

void importKradioripperSettings::importSettingsIfDesired()
{
  KConfigGroup config_group(settings_general::self()->config(), "kradioripperimport");
  if (config_group.hasKey("importRequested") &&
        config_group.readEntry<bool>("importRequested", false)) {
    importSettings();
    KMessageBox::information(0, i18nc("@info", "KRadioRipper's settings have been imported."));
  };
}

QString importKradioripperSettings::allocateStreamFile()
{
  // variables
  QFileInfo info;
  QString returnValue;

  // code
  KTemporaryFile newConfigFile;
  // Maybe directories are missing in our desired path - locateLocal() will
  // create them if nesessary...
  newConfigFile.setPrefix(helper_locateConfigfile("stream_"));
  newConfigFile.setSuffix("_rc");
  newConfigFile.open();
  newConfigFile.setAutoRemove(false);
  info.setFile(newConfigFile.fileName());
  returnValue = info.fileName();
  newConfigFile.close();
  return returnValue;
}

QString importKradioripperSettings::helper_locateConfigfile(const QString & configfileName)
{
  return KStandardDirs::locateLocal("appdata", configfileName);
}

inline QString importKradioripperSettings::helper_stationlistXmlFilepath()
{
  return KStandardDirs::locateLocal("appdata", "stationlist.xml");
}

void importKradioripperSettings::importSettings()
{
  // copy config file
  KIO::NetAccess::synchronousRun(
    KIO::file_copy(
      KUrl(KStandardDirs::locate("config", "kradioripperrc")),
      KUrl(KStandardDirs::locate("config", settings_general::self()->config()->name())),
      -1,
      KIO::HideProgressInfo|KIO::Overwrite),
    0);

  // reload config file
  settings_general::self()->readConfig();

  // convert stream list
  KConfigGroup group_streamlist(settings_general::self()->config(), "internal");
  QStringList old_list_FileFullNames =
    group_streamlist.readPathEntry("streamConfigFiles", QStringList());
  QString xmlSnippet = "    <folder name=\"KRadioRipper\">\n";
  foreach (const QString & old_fileFullName, old_list_FileFullNames) {
    QString new_fileSimpleName = allocateStreamFile();
    KIO::NetAccess::synchronousRun(
      KIO::file_copy(
        KUrl(old_fileFullName),
        KUrl(KStandardDirs::locate("appdata", new_fileSimpleName)),
        -1,
        KIO::HideProgressInfo|KIO::Overwrite),
      0);
    xmlSnippet += ("        <stream>" + new_fileSimpleName + "</stream>\n");
  };
  xmlSnippet += "    </folder>\n</stationlist>";
  group_streamlist.deleteEntry("streamConfigFiles");
  group_streamlist.sync();
  QFile xmlFile(helper_stationlistXmlFilepath());
  xmlFile.open(QIODevice::ReadWrite);
  // QTextStream will use the default locale (for example 8859-1), and not Unicode.
  // However, as our XML is plain ASCII, this doesn't matter.
  QString newXmlFileContent = QTextStream(&xmlFile).readAll().replace("</stationlist>", xmlSnippet);
  QFile xmlNewFile(helper_stationlistXmlFilepath() + ".new");
  xmlNewFile.open(QIODevice::ReadWrite);
  QTextStream(&xmlNewFile) << newXmlFileContent;
  xmlFile.remove();
  xmlNewFile.rename(helper_stationlistXmlFilepath());

  // adapt settings for next startup
  KMessageBox::saveDontShowAgainYesNo("askForKradioripperImport", KMessageBox::No);
  KConfigGroup group_import(settings_general::self()->config(), "kradioripperimport");
  group_import.writeEntry<bool>("importRequested", false);
  group_import.sync();
  settings_general::self()->writeConfig();
  settings_general::self()->readConfig();
}

void importKradioripperSettings::askForImport(bool use_dontAskAgainName)
{
  QString dontAskAgainName;
  if (use_dontAskAgainName) {
    dontAskAgainName = "askForKradioripperImport";
  }
  if (KMessageBox::questionYesNo(0,
                                 i18nc("@info", "<para>Do you want to import KRadioRipper "
                                         "settings?</para><para><note>The import will be "
                                         "performed when you next start KStreamRipper."
                                         "</note></para>"),
                                 i18nc("@title:window", "Import KRadioRipper settings"),
                                 KStandardGuiItem::yes(),
                                 KStandardGuiItem::no(),
                                 dontAskAgainName) == KMessageBox::Yes) {
    KConfigGroup group_import(settings_general::self()->config(), "kradioripperimport");
    group_import.writeEntry<bool>("importRequested", true);
    group_import.sync();
    settings_general::self()->writeConfig();
    settings_general::self()->readConfig();
  };
}
