/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QDateTime>
#include <QDir>
#include <QFile>
#include <QString>
#include <QTextStream>
#include <KAboutData>
#include <KCmdLineArgs>
#include <KDebug>
#include <KUniqueApplication>

#include "importkradiorippersettings.h"
#include "mainwindow.h"
#include "mdebug.h"
#include "propertyvalue.h"
#include "version.h"

#ifdef CUSTOMMESSAGEHANDLER
/** This function provides a custom message handler, which creates a log file
in the home directory of the user. It is only included if #CUSTOMMESSAGEHANDLER
is defined. */
void myMessageOutput(QtMsgType type, const char *msg)
{
  QFile *myfile = new QFile(QDir::homePath() + "/kstreamripper.log");
  myfile->open(QIODevice::Append);
  QTextStream stream;
  stream.setDevice(myfile);
  switch (type) {
    case QtDebugMsg:
      stream << "DEBUG: ";
      break;
    case QtWarningMsg:
      stream << "WARNING: ";
      break;
    case QtCriticalMsg:
      stream << "CRITICAL: ";
      break;
    case QtFatalMsg:
      stream << "FATAL: ";
      break;
    default:
      stream << "UNKNOWN MESSAGE TYPE: ";
      break;
  };
  stream << QString::fromUtf8(msg);
  stream << '\n';
  stream.flush();
  myfile->close();
  delete myfile;
}
#endif

/** The main function of KStreamRipper. */
int main (int argc, char * argv[])
{
  qRegisterMetaType<PropertyValue>("PropertyValue");
  qRegisterMetaType<PropertyValue::propertyValueType>("PropertyValue::propertyValueType");
  qRegisterMetaType<ripping::statusType>("ripping::statusType");
  KAboutData aboutData("kstreamripper", // internal program name
                       0, // name of the message catalog. 0: same as internal program name
                       ki18nc("@title KAboutData: displayed program name", "KStreamRipper"),
                       VERSION,
                       ki18nc("@title KAboutData: short program description",
                              "A program for recording audio streams"),
                       KAboutData::License_Unknown,  // will be overwritten later
                       ki18nc("@info:credit KAboutData: copyright hint",
                              "(c) 2008-2010 Lukas Sommer"),
                       ki18nc("@info KAboutData: further hints", "Based on Streamripper."),
                       "http://kstreamripper.sourceforge.net/",
                       // bug email address:
                       QString("%1@gmail.com").arg("SommerLuk").toLatin1());
  aboutData.addAuthor(ki18nc("@info:credit KAboutData/Author: name (do not translate this!)",
                             "Lukas Sommer"),
                      ki18nc("@info:credit KAboutData/Author: function", "Main author"),
                      QString("SommerLuk%1").arg("@gmail.com").toLatin1());
  aboutData.addCredit(ki18nc("@info:credit KAboutData/Credit: name",
                             "Greg Sharp and the Streamripper team"),
                      ki18nc("@info:credit KAboutData/Credit: function",
                             "Authors of Streamripper, which is the base for KStreamRipper"),
                      QString("gregsharp%1").arg("@users.sourceforge.net").toLatin1(),
                      "http://streamripper.sourceforge.net/");
  aboutData.addCredit(ki18nc("@info:credit KAboutData/Credit: name", "Bill Marshman"),
                      ki18nc("@info:credit KAboutData/Credit: function",
                             "Review of the English translation"),
                      QString("bill.marshman%1").arg("@gmail.com").toLatin1());
  aboutData.addCredit(ki18nc("@info:credit KAboutData/Credit: name", "Detlef Reichelt"),
                      ki18nc("@info:credit KAboutData/Credit: function", "Packages for Packman"),
                      QString("detlef%1").arg("@links2linux.de").toLatin1(),
                      "http://packman.links2linux.org/package/kstreamripper");
  aboutData.addCredit(ki18nc("@info:credit KAboutData/Credit: name", "Rajko Bock"),
                      ki18nc("@info:credit KAboutData/Credit: function",
                             "Packages for Mandriva"),
                      QString("info%1").arg("@randosweb.de").toLatin1(),
                      "http://www.randosweb.de");
  aboutData.addCredit(ki18nc("@info:credit KAboutData/Credit: name", "Alessandro Ghersi"),
                      ki18nc("@info:credit KAboutData/Credit: function", "Packages for Ubuntu"),
                      QString("alessandro.ghersi%1").arg("@fastwebnet.it").toLatin1(),
                      "http://packages.ubuntu.com/karmic/kstreamripper");
  // Set credits to translators manually to allow semantic markup.
  // (No fallback available in our .po! So don't change this.)
  aboutData.setTranslator(ki18nc("@info:credit NAME OF TRANSLATORS "
                                   "Dear translator. Enter your name here. This special "
                                   "string is used to display a credit to you at "
                                   "Help->About KStreamRipper->Translation. /// The "
                                   "expected format is a coma separated list. Example: 'First "
                                   "Translator,Second Translator,' is a list "
                                   "of 3 translators: 1: 'First Translator' 2: 'Second "
                                   "Translator' 3: no name is displayed, only email address",
                                 "Your names"),
                          ki18nc("@info:credit EMAIL OF TRANSLATORS "
                                   "Dear translator. Enter your email address here. This special "
                                   "string is used to display a credit to you at "
                                   "Help->About KStreamRipper->Translation "
                                   "Or leave it blank when you do not want to publish your email "
                                   "address. If you want, you can use %1 instead of the 'at' "
                                   "sign (for anti-spam reasons). /// The expected "
                                   "format is a coma separated list. Example: "
                                   "'first%1nospam.please,,thierd%1nospam.please' is a list of 3 "
                                   "email addresses: 1: 'first@mail.com' 2: no address (only the "
                                   "name is displayed) 3: 'thierd@nospam.please' /// Attention: "
                                   "The length of the list is not determined by _this_ string, "
                                   "but by the translation of 'Your names'!",
                                 "Your emails").subs("@"));
#if KDE_IS_VERSION(4,1,0)
  // Overrides License_Unknown, which was specified in the constructor:
  aboutData.addLicenseText(
    ki18nc(
      "@info/plain (%1 will be replaced by the 'at' sign - doing it this way for anti-spam "
        "reasons. "
        "Please use this also in the translation file.) The text is a modified version of the "
        "text that is proposed in the GPL. See http://www.gnu.org/licenses/translations.html for "
        "possibly yet available translations.",
      "Copyright (C) 2008-2010  Lukas Sommer &lt;SommerLuk%1&gt;<nl/>"
        "<nl/>"
        "This program is free software; you can redistribute it and/or<nl/>"
        "modify it under the terms of the GNU General Public License as<nl/>"
        "published by the Free Software Foundation; either version 2 of<nl/>"
        "the License or (at your option) version 3 or any later version<nl/>"
        "accepted by the membership of KDE e.V. (or its successor approved<nl/>"
        "by the membership of KDE e.V.), which shall act as a proxy<nl/>"
        "defined in Section 14 of version 3 of the license.<nl/>"
        "<nl/>"
        "This program is distributed in the hope that it will be useful,<nl/>"
        "but WITHOUT ANY WARRANTY; without even the implied warranty of<nl/>"
        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<nl/>"
        "GNU General Public License for more details.<nl/>"
        "<nl/>"
        "You should have received a copy of the GNU General Public License<nl/>"
        "along with this program.  If not, "
        "see &lt;http://www.gnu.org/licenses/&gt;.").subs("@gmail.com"));
#endif
  KCmdLineArgs::init(argc, argv, &aboutData);
  KUniqueApplication::addCmdLineOptions();
  if (!KUniqueApplication::start()) {
    mDebug() << "Will return 0...";
    return 0;
  };
  KUniqueApplication app;
  // NOTE Starting from here we can use mDebug.
#ifdef CUSTOMMESSAGEHANDLER
  qInstallMsgHandler(myMessageOutput);
  mDebug()
    << "\n#\n#\n#\n#\nStarting debug log at"
    << QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss.zzz")
    << "with KStreamRipper"
    << VERSION;
#endif
  mDebug() << "Will import KRadioRipper settings if desired...";
  importKradioripperSettings::importSettingsIfDesired();
  mDebug() << "Will test if we restore the session...";
  if (app.isSessionRestored()) {
    mDebug() << "Will restore session...";
    kRestoreMainWindows< MainWindow >();
    mDebug() << "Done restoring session...";
  } else {
    mDebug() << "Will NOT restore session...";
    mDebug() << "Will instanciate MainWindow...";
    MainWindow * window = new MainWindow();
    mDebug() << "MainWindow instance has been created at" << window;
    mDebug() << "Will set object name of the MainWindow instance...";
    window->setObjectName("KStreamRipperMainWindow#");  // '#' will be replaced with unique numbers
    mDebug() << "Object name has been set to" << window->objectName();
    mDebug() << "Will call MainWindow::show()...";
    window->show();
    mDebug() << "Done NOT restoring session...";
  };
  mDebug() << "Will start the main event loop...";
  return app.exec();
  mDebug() << "event loop has finished.";
}
