/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "stationlistwidget.h"
#include <QCloseEvent>
#include <QDockWidget>
#include <QPointer>
#include <KAction>
#include <KSystemTrayIcon>
#include <KToggleAction>
#include <KXmlGuiWindow>
#include "streamdirectorydockwidget.h"

/** \brief The mainwindow.
*
* This class provides the mainwindow and handels the actions, session
* management, the global setting dialog, the <i>Tip of the day</i> and so on.
*
* \note The desctructor of this function will not necesarily be called, so also the children
* objects will not necesarrily be destroyed. So don't relay on the desctructor of children
* objects to save data, the application state and so on! */
class MainWindow : public KXmlGuiWindow
{

    Q_OBJECT

  public:
    /** The constructor.
    *
    * <ul>
    * <li>Constructs the central widget (see #m_streamlistwidget).</li>
    * <li>Sets up the actions.
    *     <ul>
    *     <li>Makes the actions availeble to the XML-GUI framework
    *         (See <tt>KXmlGuiWindow</tt> documentation).</li>
    *     <li>Makes the actions availeble in the context menu of the
    *         central widget (see stationlistWidget).</li>
    *     </ul></li>
    * <li>Set up the XML GUI.</li>
    * <li>Display <i>Tips of the Day</i> (if not disabled by the user).</li>
    * </ul> */
    MainWindow(QWidget *parent=0);
    /** The destructor.
    *
    * This is a dummy function. See #prepareForExitingApp() for details. */
    virtual ~MainWindow();

  public slots:
    /** Reimplemented from base class to influence also floating dock widgets. */
    virtual void setVisible(bool visible);

  protected:
    /** Reimplemented from base class. */
    virtual void closeEvent(QCloseEvent *event);
    /** Reimplemented from KMainWindow. Saves some settings before the application closes. */
    virtual bool queryClose();
    /** Reimplemented from KMainWindow to restore the last session. */
    virtual void readProperties(const KConfigGroup & m_configGroup);
    /** Reimplemented from KMainWindow to save this session for restoring. */
    virtual void saveProperties(KConfigGroup & m_configGroup);

  private:
    /** A pointer to the central widget.
    *
    * The central widget is created in the constructor
    * of this class, and the constructor also sets
    * this pointer.
    *
    * To access the central widget, we could use
    * QMainWindow::centralWidget() and typecast to
    * stationlistWidget - but using this pointer,
    * we don't have to typecast all the time! */
    QPointer<stationlistWidget> m_streamlistwidget;
    /** A pointer to the dock widget that contains the stream directory. */
    QPointer<streamdirectoryDockwidget> m_streamDirectory;
    /** The action that manages if the stream directory is displayed or not. */
    QPointer<KToggleAction> showStreamdirectoryAction;
    /** The system tray support. */
    KSystemTrayIcon systray;

  private slots:
    /** Actualizes the status in the status bar and in the tooltip of the system tray icon. */
    void actualize_systray_and_statusbar();
    /** Asks if the user wants to imports KRadioRipper's settings. */
    void askForKradioripperImport();
    /** Displays the settings_general_dialog. */
    void display_global_settings_dialog();
    /** Displays the tip of the day (independently from if the user
    *   has disabled them or not). */
    void displayTipOfDay();
    /** Convenience slot for #showStreamdirectory(bool false). */
    void hideStreamdirectory();
    /** Calls #queryClose().
    *
    *   This is necessary because queryClose() is not always called by the framework. For example,
    *   when you use the menu (File->Quit) or the system tray (right click->Quit) this results in
    *   a call of <tt>KApplication.quit()</tt> which will exit the event loop without closing
    *   regulary the main window - it will not even destroy it.
    *
    *   \note This slot is connected the signal <tt>QCoreApplication::aboutToQuit()</tt> and is
    *   called immediately before leaving the event loop. So, in this function, we can not relay on
    *   a running event loop! Note also that the signal is not always emitted - when using session
    *   management for example it is not emitted. But the calls of #queryClose() and are done
    *   automatically and the calls of this slot together seem to make sure that #queryClose() is
    *   called always at least \e one time. */
    void prepareForExitingApp();
    /** Sets up the actions that are supported by this class. */
    void setupActions();
    /** Sets up the dock widget with the stream directory. */
    void setupStreamDirectory();
    /** Sets up the system tray icon and the status bar. */
    void setupSystrayAndStatusbar();
    /** Sets the visibility of the streamdirectory and saves this value also in the config file.
    * @param visible Choose \e true to show the streamdirectory and \e false to hide it.
    * \note If #MainWindow is not visible, the streamdirectory will be hidden indepentendly of
    * the parameter. */
    void showStreamdirectory(bool visible);
    /** Toggles the property "visibility" of QMainWindow. */
    void toggleVisibility(
      const QSystemTrayIcon::ActivationReason reason = QSystemTrayIcon::Trigger);
};

#endif
