/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STATIONLISTITEM_H
#define STATIONLISTITEM_H

#include <QList>
#include <QPointer>
#include <QString>
#include "radiostation.h"

/** \brief An item for #stationlistModel
*
* This class provides item objects for #stationlistModel that can be organized in a
* tree structure. It has a determined #itemType available through #type().
*
* \sa #streamDirectoryModel */
class stationlistItem
{

  public:
    /** An enumeration that identifies the type of this item.
    * \li \e folder This is a folder item and it can contain child items.
    * \li \e stream This is a stream item and it can not have child items. */
    enum itemType {
      folder,
      stream
    };
    /** Constructs a folder item.
    * @param folderName The name of the folder */
    explicit stationlistItem(const QString &folderName);
    /** Constructs a stream item.
    * @param streamObject The stream object
    * \note The stream object will be deleted in the desctructor of this class! */
    explicit stationlistItem(radioStation *streamObject);
    /** The destructor.
    *
    * Will delete all child items.
    *
    * \note The #radioStation objects will \e not be deleted! */
    virtual ~stationlistItem();
    /** If \e this object is of type #folder, this function makes another object become a child
    * of \e this object. Otherwise, it does nothing and returns \e false.
    * @param child A pointer to the new child.
    * @returns Whether the operation has been performed.
    * \warning Don't call this function with a child that has yet a parent! */
    bool appendChild(stationlistItem *child);
    /** \returns A pointer to the child entry at the specified row (or \e NULL if the row doesn't
    * exist).
    * @param row The row number for which the pointer is requested. */
    stationlistItem *child(int row) const;
    /** \returns The number of children of this object. */
    int childCount() const;
    /** Detaches this item from its parent item (removes it from parent's child list and
    *   sets its own #parentItem to 0). It is save to execute this function on objects
    *   that have no parent. */
    void detach();
    /** @returns The folder name of this item (or an empty string
    * if this is not a #folder item).
    * \note Test the type of this object with #type() before using this function! */
    QString folderName() const;
    /** If \e this object is of type #folder, this function makes another object become a child
    * of \e this object. Otherwise, it does nothing and returns \e false.
    * @param child A pointer to the new child.
    * @param row The row where the new child is inserted. If to small, it will be inserted at
    *            position 0, if to big it will be appended.
    * @returns Whether the operation has been performed.
    * \warning Don't call this function with a child that has yet a parent! */
    bool insertChild(stationlistItem *child, int row);
    /** \returns A pointer to the parent of this object (or \e NULL if it has no parent). */
    stationlistItem *parent() const;
    /** \returns The row number that this object has within it's parent (or 0 if it
    * has no parent). */
    int row() const;
    /** Changes the folder name. */
    void setFolderName(const QString & folderName);
    /** @returns The stream object of this item (or NULL if the object has been deleted
    * or if this is not a #stream item).
    * \note Test the type of this object with #type() before using this function! */
    radioStation *streamObject() const;
    /** @returns The type of this object. */
    stationlistItem::itemType type() const;

  private:
    /** The list of child items. */
    QList<stationlistItem *> childItems;
    /** Initializes the object. */
    void initialize();
    /** The folder name of this item. */
    QString m_folderName;
    /** The type of this object. */
    itemType m_itemType;
    /** A pointer to the #radioStation object. */
    QPointer<radioStation> m_radiostation;
    /** A pointer to the parent of this object (or \e NULL if this object doesn't have a
    * parent). */
    stationlistItem *parentItem;
    /** A pointer to \e this. */
    stationlistItem *self;
};

#endif
