/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STATIONLISTMODEL_H
#define STATIONLISTMODEL_H

#include <QAbstractItemModel>
#include <QList>
#include <QPointer>
#include <QXmlStreamWriter>
#include <phonon/mediaobject.h>
#include <phonon/path.h>
#include <phonon/audiooutput.h>
#include "folderpropertiesdialog.h"
#include "listensupport.h"
#include "radiostation.h"
#include "stationlistitem.h"

class stationlistWidget;

/** \brief A model (for model-view-programming) handling radio streams.
*
* This class handels radio stations. It reads the list of configuration
* files of radio stations from \e settings_general and represents these radio
* stations. Internally, it uses objects of the class \e radioStation
* to handle the stations. This class can also create new radio stations or
* delete existing ones - and takes care of creating/changing/removing the
* corresponding config files.
*
* This class inherits from \e QAbstractTableModel (and this way indirectly from
* \e QAbstractItemModel). So it can be used as model for Qt's model-view-programming.
* Use a \e QTableView object to watch the data of this model.
*
* \warning Use only one instance of this class at the same time! (Otherwise, the
* same stream could be recorded by more than one instance at the same time - and
* written by all instances to the same file! That's bad! Also this class uses the config
* files - and it is bad when more than one instance uses them...)
*
* \warning You can use only \e one view to show the content of this model! */
class stationlistModel : public QAbstractItemModel
{

     Q_OBJECT

     /** This property holds the total bandwidth that is actually
     *   really used by the streams (summation of the bitrate of all streams
     *   which are not ripping::idle).
     *
     * \warning The formated version contains the value in kbit/s (with SI prefix which means
     * 1000 bit/s, <em> not 1024 bit/s</em>).
     *
     * \li \e read: \code quint64 bandwidth() const \endcode
     * \li \e notify: \code void bandwidthChanged() \endcode
     *
     * \sa \code void recalculate_numberOfActiveStreams_and_bandwidth() \endcode
     * \sa \code quint64 internal_bandwidth \endcode */
     Q_PROPERTY(quint64 bandwidth READ bandwidth)

     /** This property gives access to a duplicate-free list of all actually selected streams
     * (either selected directly or because one of its parent folders is selected).
     *
     * The value is always keept up to date.
     *
     * \li \e read: \code QList<radioStation *> selectedStreams() const \endcode
     * \li \e notify: \code void selectedStreamsChanged() \endcode
     *
     * \note You should connect to #selectedStreamsChanged() instead of connecting to
     * the signals of <tt>QItemSelectionModel</tt> because this makes sure that at the
     * time of the slot call, all properties are yet updated.
     *
     * \sa \code void recalculateSelectedStreams() \endcode
     * \sa \code QList<radioStation *> internal_selectedStreams \endcode */
     Q_PROPERTY(QList<radioStation *> selectedStreams READ selectedStreams)

     /** This property holds the number of active streams (number of all streams
     *   which are not ripping::idle).
     *
     * \li \e read: \code int numberOfActiveStreams() const \endcode
     * \li \e notify: \code void numberOfActiveStreamsChanged() \endcode
     *     and also, if the new number is 0:
     * \li \e notify: \code void numberOfActiveIsZero() \endcode
     *
     * \sa \code void recalculate_numberOfActiveStreams_and_bandwidth() \endcode
     * \sa \code int internal_numberOfActiveStreams \endcode */
     Q_PROPERTY(int numberOfActiveStreams READ numberOfActiveStreams)

  public:
     /** The constructor.
     * \param parent The parent of this object. This is important for the necessary
     * interaction between model and view (for example: selection handling).
     * \param mainWidget The widget to which configuration dialogs will be centered.
     * \note This model sets itself with <tt>setModel()</tt> in <tt>parent</tt>. */
     explicit stationlistModel(stationlistWidget *parent,
                               QWidget *mainWidget = 0);

     /** The destructor. */
     virtual ~stationlistModel();

     /** This enum is used in columnInfo() to determinate which action
     * will be performed:
     * \li \e stationlistModel::columnHeaderTitle:
     *     Get the header title of the given column (QString).
     * \li \e stationlistModel::columnHeaderToolTip:
     *     Get the tooltip of the header of the given column (QString).
     * \li \e stationlistModel::columnHeaderWhatsThis:
     *     Get the "What's this"-help of the header of the given column (QString).
     * \li \e stationlistModel::columnWidth:
     *     Get the width of the given column from settings_general (quint64). A view
     *     for this model should take care of this value and read it and apply it in
     *     his constructor. In the config file the values aren't saved as a simple list
     *     of values (ordered by column number), but each column has an individual entry
     *     name for the width. This way it is guaranteed that, also when the model changes
     *     the order of columns, each column guards its width. Furthermore, no changes at
     *     the "view" widget are necessary as this system works without that the view
     *     widget knows about the order of the columns and of there content.
     * \li \e stationlistModel::setColumnWidth:
     *     Set the width of the given column in settings_general. A view for this model
     *     should write this value back in his contructor to guarantee that the last state
     *     of the UI is saved.
     * \li \e stationlistModel::columnVisibility:
     *     Get the visibility of the given column from settings_general (bool). A view
     *     for this model should take care of this value and read it and apply it in his
     *     constructor. Like stationlistmodel::columnWidth, it is saved individually for
     *     each column.
     * \li \e stationlistModel::setColumnVisibility:
     *     Set the visibility of the given column in settings_general. A view for this
     *     model should write this value back in his contructor to guarantee that the
     *     last state of the UI is saved.
     * \li \e stationlistModel::columnData:
     *     Get the data of the field specified by column and row (#PropertyValue). */
     enum columnInfoType{
       columnHeaderTitle,
       columnHeaderToolTip,
       columnHeaderWhatsThis,
       columnWidth,
       setColumnWidth,
       columnVisibility,
       setColumnVisibility,
       columnData
     };

     /** See property #bandwidth. */
     quint64 bandwidth() const;

     /** Reimplemented from QAbstractTableModel.
     * @param parent As this is a table model (non-hirarchical!),
     *               this parameter is ignored.
     * \returns the number of columns */
     virtual int columnCount(const QModelIndex & parent = QModelIndex()) const;

     /** This function is the central place to access all information
     *   about the data in the model and also about related things (column
     *   width, visibility, header title...).
     *
     *   \warning This function has the \e const qualifier, but this doesn't
     *   mean that nothing changes. You have not only \e read access, but
     *   also \e write access (to some values).
     *
     *   \param type Specifies the type of action that will be performed.
     *   See #columnInfoType for details.
     *
     *   \param column The column to access.
     *
     *   \param stream The stream to access. Is ignored if the action is independent
     *   from the stream.
     *
     *   \param value The value to write. Is ignored if the action is not
     *   a writing action.
     *
     *   \returns The requested data as QVariant. If the passed arguments
     *   were invalid or if it was a write operation, than an invalid
     *   QVariant is returned. */
     QVariant columnInfo(const columnInfoType type,
                         const int column,
                         const radioStation *stream = 0,
                         const quint64 value = (-1)) const;

     /** Reimplemented from QAbstractTableModel.
     *
     * Uses columnInfo() to deliver the data.
     *
     * @param index As this is a table model (non-hirarchical!),
     *              only rows and columns of the index are relevant.
     *              The parent is ignored. If the index doesn't belong
     *              to this model, an invalid QVariant is returned.
     * @param role  The type of information that is requested. Supported are
     *              \li Qt::DisplayRole (the data itself)
     *              \li Qt::ToolTipRole
     *              \li Qt::WhatsThisRole.
     *
     * For other values, an invalid \e QVariant is returned.
     * \returns the data of a the spezified item.*/
     virtual QVariant data (const QModelIndex & index, int role = Qt::DisplayRole) const;

     /** Reimplemented function that provides support for dropping URL lists via "drag and drop. */
     virtual bool dropMimeData(const QMimeData *data,
                               Qt::DropAction action,
                               int row,
                               int column,
                               const QModelIndex & parent);

     /** Reimplemented function that says that dropping streams via "drag and drop" is accepted
     * at \e each place in the view widget. */
     virtual Qt::ItemFlags flags(const QModelIndex & index) const;

     /** Reimplemented from QAbstractTableModel.
     *
     * Uses columnInfo() if orientation is horizontal. */
     virtual QVariant headerData(int section, Qt::Orientation orientation, int role) const;

     /** Reimplemented from base class. */
     virtual QModelIndex index (int row,
                                int column,
                                const QModelIndex & parent = QModelIndex()) const;

     /** With this object, we can listen in streams.
     * \note This object is handled automatically. So don't make changes to this object. However,
     * it may make sense to connect to its signals to obtain information. */
     listenSupport listen;

     /** Reimplemented from base class. */
     QMimeData *mimeData(const QModelIndexList & indexes) const;

     /** Reimplemented from Qt. Provides a list of the mime types that are accepted for dropping
     * streams via "drag and drop". */
     virtual QStringList mimeTypes() const;

     /** See property #numberOfActiveStreams. */
     int numberOfActiveStreams() const;

     /** Reimplemented from base class. */
     virtual QModelIndex parent(const QModelIndex & index) const;

     /** - Remembers a list of running streams (which is necessary if we do session
     *     management)
     *
     *   - Shuts them down. As this can take a while, a "busy" dialog is displayed.
     *
     * \sa #saveProperties()
     * \sa #rememberListOfStreamsWhichTheUserWantsToRip_ifNotYetDone() */
     virtual bool queryClose();

     /** Reads properties for restoring the previous session
     * (using session management).
     * @param m_configGroup The location where to read the properties from
     * \sa #saveProperties() */
     virtual void readProperties(const KConfigGroup & m_configGroup);

     /** Reimplemented from QAbstractItemModel.
     *
     *   Removes \e count radio stations starting with the given row (radio station) from
     *   this model and deletes the corresponding config files.
     *   @param row You must pass the index (= the row) of the first stream which you want to
     *              delete. When you pass an invalid index, nothing happens.
     *   @param count number of streams that will be deleted.
     *   @param parent is ignored.
     *   \returns whether the removing was sucessful */
     virtual bool removeRows(int row, int count, const QModelIndex & parent = QModelIndex());

     /** Reimplemented from QAbstractTableModel.
     * @param parent As this is a table model (non-hirarchical!),
     *               this parameter is ignored.
     * \returns the number of rows */
     virtual int rowCount(const QModelIndex & parent = QModelIndex()) const;

     /** Saves properties for restoring this session later
     * (using session management).
     * @param m_configGroup The location where to save the properties
     * \sa #readProperties()
     * \sa #rememberListOfStreamsWhichTheUserWantsToRip_ifNotYetDone()
     * \sa #m_listOfStreamsOfWhichTheUserWantsThatTheyRip */
     virtual void saveProperties(KConfigGroup & m_configGroup);

     /** See property #selectedStreams. */
     QList<radioStation *> selectedStreams() const;

     /** Reimplemented from QAbstractTableModel.
     *
     * This is just a dummy function. It (still) does nothing. */
     virtual void sort (int column, Qt::SortOrder order = Qt::AscendingOrder);

  signals:
     /** See property #bandwidth. */
     void bandwidthChanged();
     /** See property #numberOfActiveStreams. */
     void numberOfActiveStreamsChanged();
     /** See property #numberOfActiveStreams. */
     void numberOfActiveStreamsIsZero();
     /** Is emitted always when one of the streams changes its relay port. */
     void relayportChanged();
     /** See property #selectedStreams. */
     void selectedStreamsChanged();
     /** Is emitted always when one of the streams changes its status or its error. */
     void statusChanged();

  public slots:
     /** This function adds a new folder: It displays a configuration dialog
     *   for the new folder. If accepted by the user (= clicking OK), the new
     *   folder is inserted in the root folder.) */
     void addNewFolder();
     /** This function adds a new folder: It displays a configuration dialog
     *   for the new folder. If accepted by the user (= clicking OK), the new
     *   folder is inserted in the actually selected folder. (Works only if actually
     *   exactly \e one item is selected and this item is a folder. Otherwise, the
     *   new folder is simply inserted in the root folder.) */
     void addNewFolderInFolder();
     /** This function adds a new radio station: It displays a configuration dialog
     *   for the new stream. If accepted by the user (= clicking OK), the new
     *   stream is inserted in the root folder. */
     void addNewStation();
     /** This function adds a new radio station: It displays a configuration dialog
     *   for the new stream. If accepted by the user (= clicking OK), the new
     *   stream is inserted in the actually selected folder. (Works only if actually
     *   exactly \e one item is selected and this item is a folder. Otherwise, the
     *   new stream is simply inserted in the root folder.) */
     void addNewStationInFolder();
     /** Enables the listening in to the actually selected stream.
     * \note If there is no stream selected of more than 1 stream are
     * selected, than listening in is disabled. */
     void enableListeningIn();
     /** Disables listening in. */
     void disableListeningIn();
     /** Pastes the content of the clipboard (using #dropMimeData()). */
     void paste();
     /** Pastes the content of the mouse selection (using #dropMimeData()).
     * @param index The position where the mouse selection will be inserted. */
     void pasteSelection(const QModelIndex & index);
     /** This slot tries to start the recording of all selected stream.
     * Selected streams means: directly selected or one of its parent folders is
     * selected. Trying means: when the stream is actually shutting down, than the
     * call will be ignored by #radioStation. */
     void record();
     /** If exactly one stream or folder is selected, then this slot shows the config dialog.
     *   Else nothing happens. */
     void showConfigDialog();
     /** This slot stops the recording of all selected stream.
     * Selected streams means: directly selected or one of its parent folders is
     * selected. */
     void stopRecording();

  protected:
     /** A pointer to the view that shows this model.
     * \warning This pointer can be 0! Test this before using it! */
     QPointer<stationlistWidget> view;

  private:
     /** Internal enum. */
     enum mode {
       add,
       move
     };
     /** Helps to create a new folder. Displays a settings dialog for the folder that will
     * be created. If accepted, the folder will be inserted. If index is a folder, it will
     * be added in this folder. If not, it will be added in the root folder. */
     void helper_addNewFolder(const QModelIndex & index);
     /** Helps to create a new station. Displays a settings dialog for the station that will
     * be created. If accepted, the station will be inserted. If index is a folder, it will
     * be added in this folder. If not, it will be added in the root folder. */
     void helper_addNewStation(const QModelIndex & index);
     /** This helper function sets up all the necessary connection of
     * signals and slots which are needed to keep this model up to date.
     * \warning Call this function only \e once for each station.
     * \param m_stream a pointer to the station for which you want
     * to set up the connection */
     void helper_connectSignalsAndSlots(radioStation *m_stream);
     /** Extracts a list of URLs from mime data. */
     QList<QUrl> helper_convertToUrl(const QMimeData *data);
     /** Creates an item for a given stream, adjusts #radioStation::index, and
     * connects the signals of #radioStation to the slots of this class.
     * @param stream A #radioStation object
     * @returns A pointer to a newly created item. */
     stationlistItem *helper_createStreamItem(radioStation *stream);
     /** Deletes the #radioStation object and removes its config files from disk.
     * @param station A pointer to the station (must be on the heap) */
     void helper_deleteRadiostationobjectAndRemoveConfigfiles(radioStation *station);
     /** Deletes the #radioStation objects of this respective of all its direct and
     * indirect child items. \note The #stationlistItem obejcts themself are not deleted.
     * @param item A pointer to the item
     * \warning Execute this function only on items that are \e not part of the tree! */
     void helper_deleteRadiostationobjectAndRemoveConfigfiles(stationlistItem *item);
     /** Overloaded function that supports to compare items directly. See the other function
     * for details. This one, as it permits folders to be compared, always says that folder
     * are \e before streams when comparing a folder with a stream. When comparing two folders
     * than it uses alphabetical order, following the argument \e order. */
     bool helper_firstIsAfterSecond(stationlistItem *firstItem,
                                    stationlistItem *secondItem,
                                    int column,
                                    Qt::SortOrder order);
     /** This function compares 2 values for sorting. It uses the following
     * sort order:
     * -# #PropertyValue::unset
     * -# #PropertyValue::error
     * -# #PropertyValue::value in order (following #PropertyValue.internalValue
     *    interpretated as integer if possible, otherwise case-insensitive string
     *    sorting of #PropertyValue.formatedValue)
     *
     * If both values have
     * the same order, it returns always \e true.
     * \returns If the first item should be after the second item when sorting. */
     bool helper_firstIsAfterSecond(const PropertyValue & firstValue,
                                    const PropertyValue & secondValue,
                                    Qt::SortOrder order);
     /** @returns If the item is a stream or has at least one stream as direct or indirect
     * child. */
     bool helper_hasStreams(stationlistItem *item);
     /** @param item The item for which you want to obtain information. It is okay to pass \e 0
     *   (will return \e false) or a pointer to the root item (will return \e true). However, an
     *   invalid pointer will crash!
     *   @returns If the item is actually part of the model.
     *   \note The item must have a valid parent or \e 0 as parent - otherwise a crash will
     *   occur. */
     bool helper_itemIsPartOfModel(stationlistItem *item);
     /** @param parentItem The base item
     * @returns If parentItem is a stream, it returns a list that just contains this item.
     * If parentItem is a folder, it returns a list that contains all its direct
     * and indirect child items that are streams.
     * \warning This function will crash if the item is invalid. */
     QList<radioStation *> helper_listOfAllStreams(stationlistItem* parentItem) const;
     /** Inserts items to the model, handling also <tt>beginInsertRows</tt> and
     *   <tt>endInsertRows</tt>.
     * @param itemList The list of the items that will be added. These items can not be yet part
     * of the model!
     * @param parent The parent in which the items will be inserted. If invalid, that the function
     * will insert to the root item.
     * @param row The row number within the parent where the new items will be inserted.
     * \note This function does \e not call #helper_writeStationList. You have to do this manually!
     * \warning This function is \e very fast, but does no checks. 'row' must be a
     * valid row numbers within the parent.
     * \sa #helper_removeItems() */
     void helper_insertItems(QList<stationlistItem *> itemList,
                             const QModelIndex &parent,
                             int row);
     /** \sa
     * #helper_insertItems(QList<stationlistItem *> itemList, const QModelIndex &parent, int row)*/
     void helper_insertItem(stationlistItem * item, const QModelIndex &parent, int row);
     /** @param child The child item to test
     *   @param parent The parent item to test
     *   @returns \e true if \e child is a direct or indirect child of \e parent,
     *            otherwise \e false */
     bool helper_isInTree(stationlistItem *child, stationlistItem *parent);
     /** \returns Whether the value is of numeric type (either an integer number
     * or an floating point number) or can be converted to a numeric type
     * (#ripping::statusType).
     * \sa #helper_toLongLong */
     bool helper_isNumericValue(const QVariant & value);
     /** Moves the respective items within the tree model.
     * @param oldParent The parent of the items that will be moved.
     * @param first The row number of the first child that will be moved.
     * @param last The row number of the last child that will be moved.
     * @param newParent The parent to which the items will be moved.
     * @param newRow The row number where the items will be inserted.
     * \warning This function does not any checks! Be careful with the parameters.
     * For details:
     * \sa #helper_removeItems
     * \sa #helper_insertItem */
     void helper_moveItems(const QModelIndex &oldParent,
                           int first,
                           int last,
                           const QModelIndex &newParent,
                           int newRow);
     /** Removes items from the model, handling also <tt>beginRemoveRows</tt> and
     *   <tt>endRemoveRows</tt>.
     * @param parent The parent of the items that will be removed.
     * @param first The row number of the first item to be removed (within the parent)
     * @param last The row number of the last item to be removed (within the parent)
     * @returns A list of pointers to the item objects. The item objects still exist, but are
     * no longer part of the model.
     * \warning This function is \e very fast, but does no checks. 'first' and 'last' must be a
     * valid row numbers within the parent, and 'last' must be >= 'first'. Otherwise, the results
     * are undefined.
     * \sa #helper_insertItems() */
     QList<stationlistItem *> helper_removeItems(const QModelIndex &parent, int first, int last);
     /** If \e modelIndexList is not empty, than this function deletes the actual selection,
     * selects the given rows, if necessary expands it's parents and scrolls to them.
     * @param modelIndexList The list of rows. (One index per row is enough.) */
     void helper_selectAndScrollTo(const QModelIndexList & modelIndexList);
     /** @returns The full path to the XML file that contains the station list. */
     QString helper_stationlistXmlFilepath();
     /** Converts a #PropertyValue.internalValue to a \e qlonglong if possible
     * (integer values or #ripping::statusType).
     * \sa #helper_isNumericValue */
     qlonglong helper_toLongLong(const PropertyValue & value);
     /** Writes the actual list of streams back to the configuration file "stationlist.xml". */
     void helper_writeStationList();
     /** @param index A model index
     * @returns A pointer to the corresponding item.
     * \warning This function will return invalid data if you execute it
     * with indexes that are not part of this model. */
     stationlistItem *item(const QModelIndex & index) const;
     /** Used internally to cache the value of the property #bandwidth. */
     quint64 internal_bandwidth;
     /** Used internally to cache the value of the property #numberOfActiveStreams. */
     int internal_numberOfActiveStreams;
     /** Used internally to cache the value of the property #selectedStreams. */
     QList<radioStation *> internal_selectedStreams;
     /** Here you can save a list of the streams for which
     * #radioStation::doesTheUserWantsThatTheStreamIsRipping() is \e true. Used for session
     * management handling.
     *
     * By initialization, this pointer is \e null.
     * \warning Keep this pointer always valid and set it back to \e null when deleting the
     * QStringList. The destructor will delete a possibly remaining QStringlist, and this
     * will lead to a crash when the pointer is invalid.
     * \sa #rememberListOfStreamsWhichTheUserWantsToRip_ifNotYetDone() has details. */
     QStringList *m_listOfStreamsOfWhichTheUserWantsThatTheyRip;
     /** A pointer to the widget to which configuration dialogs should be centered.
     * This member is initialized in the constructor. */
     QPointer<QWidget> m_mainWidget;
     /** @param item An item of our tree structure
     * @param column The column of the item
     * @returns A \e QModelIndex for the respective column in the item */
     QModelIndex modelindexOfItem(stationlistItem *item, int column);
     /** A pointer to the root item of the tree structure. */
     stationlistItem *rootItem;
     /** Sorts the child items of a given parent.
     * @param parent The parent.
     * @param column The column following to which we will sort the children.
     * @param order The sort order. */
     void sortChildItems(stationlistItem *parent, int column, Qt::SortOrder order);
     /** Writes the XML elements for a given item.
     * @param item The item that has to be saved
     * @param file The file to which the XML data will be written */
     void writeItemToFile(stationlistItem *item, QXmlStreamWriter &file);

  private slots:
     /** This slot recalculates the values for the properties
     * #bandwidth and #numberOfActiveStreams from scratch. It
     * caches them in the corresponding member. If for one of
     * the properties or for both of them, the new value differs
     * from the old value, the corresponding signal(s) is/are
     * emitted.
     *
     * <i>We recalculate the value each time from scratch. We
     * could have implemented a solution the reacts on signals
     * of the radiostation objects that the status has changed,
     * and than increment or decrement our properties. This
     * solution would be more efficient, but also more complicate:
     * We would have to worry about a correct initialization,
     * about session management.... The situation would be
     * confusing and complicate. So it's better to have a clean
     * solution, also when it's not so efficient.</i> */
     void recalculate_numberOfActiveStreams_and_bandwidth();
     /** Recalculates the value of the property #selectedStreams. */
     void recalculateSelectedStreams();
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadBitrate(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadDataSize(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadMetaInterval(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadMetaInterval_milliSeconds(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadRelayPort(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadServerName(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadSong(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadStatus(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadStreamName(void *stationIndex);
     /** Used internally to emit the signal <tt>dataChanged</tt>. */
     void reloadUri(void *stationIndex);
     /** If the application is closed using session management, it is
     * undefined, if #saveProperties() or #queryClose() is executed
     * first. Because #queryClose() shuts the streams down, if
     * #saveProperties() is executed later it would not be able
     * to know which streams were running.
     *
     * To solve this problem, #queryClose() and #saveProperties() call
     * this function (in the case of #queryClose() of course \e before
     * shutting down streams). This function writes a list of running
     * streams to the member
     * #m_listOfStreamsOfWhichTheUserWantsThatTheyRip - but only if
     * there is not yet a list. This way, #saveProperties() can call
     * this function and be sure to have now a correct list of running
     * streams at #m_listOfStreamsOfWhichTheUserWantsThatTheyRip,
     * indepentently from the execution order of #saveProperties()
     * and #readProperties().
     * \sa #saveProperties()
     * \sa #queryClose()
     * \sa #m_listOfStreamsOfWhichTheUserWantsThatTheyRip */
     void rememberListOfStreamsWhichTheUserWantsToRip_ifNotYetDone();

};

#endif
